/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.ArrayList;
import java.util.List;

/**
 * This is the abstract base class for freeMule services. It contains a
 * framework for synchronized stopping of the {@link #work()} method and a basic
 * implementation for notifying {@link ServiceListener}s. General contract for
 * an AbstractService is:
 * </p>
 * <ol>
 * <li>Instantiate it using the default constructor.</li>
 * <li>Call {@link #init()}.</li>
 * <li>Set all required parameters, add listeners, etc.</li>
 * <li>Call {@link #start()} to start the main service thread.</li>
 * </ol>
 * <p>
 * To stop it, use the following sequence:
 * </p>
 * <ol>
 * <li>Call {@link #stop()}. This method will block until the main thread is
 * finished.</li>
 * <li>Call {@link #destroy()} to free all resources help by the
 * AbstractService.</li>
 * </ol>
 * <p>
 * If you want to re-use the AbstractService within the same lifecycle of the
 * VM, simply start the init sequence above at #2 (calling {@link #init()}).
 * </p>
 * <p>
 * <strong>Implementation notes:</strong> if you create a new service by
 * extending this class, remember to call <code>super.init()</code>,
 * <code>super.start()</code>, <code>super.stop()</code>, and
 * <code>super.destroy()</code> in your implementations of {@link #init()},
 * {@link #start()}, {@link #stop()}, and {@link #destroy()}!
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: AbstractService.java 130 2006-02-18 18:27:11Z bombe $
 */
public abstract class AbstractService implements Service, Runnable {

	/**
	 * Object that is used for syncing purposes.
	 */
	protected final Object syncObject = new Object();

	/**
	 * Whether the thread this service runs in should stop.
	 */
	protected volatile boolean stopping = false;

	/**
	 * All registered {@link ServiceListener}s.
	 */
	private volatile List<ServiceListener> serviceListeners;

	/**
	 * How long to wait between executions of {@link #work()}. In milliseconds.
	 */
	private long waitPeriod;

	/**
	 * The number of threads that should be running.
	 */
	private int threadCount;

	/**
	 * The {@link ThreadGroup} for all threads of this service.
	 */
	private ThreadGroup threadGroup;

	/**
	 * The currently running threads.
	 */
	private List<ServiceThread> threads;

	/**
	 * Counter for naming threads.
	 */
	private int threadCounter;

	/**
	 * Returns the full name of this service. The name is used to construct
	 * thread names.
	 * 
	 * @return The full name of this service
	 */
	public abstract String getName();

	/**
	 * Adds <code>serviceListener</code> to the list of registered
	 * {@link ServiceListener}s.
	 * 
	 * @param serviceListener
	 *            The {@link ServiceListener} to add
	 */
	public void addServiceListener(ServiceListener serviceListener) {
		if (!serviceListeners.contains(serviceListener)) {
			serviceListeners.add(serviceListener);
		}
	}

	/**
	 * Removes <code>serviceListener</code> from the list of registered
	 * {@link ServiceListener}s. Nothing will happen if
	 * <code>serviceListener</code> is not a registered
	 * {@link ServiceListener}.
	 * 
	 * @param serviceListener
	 *            The {@link ServiceListener} to remove
	 */
	public void removeServiceListener(ServiceListener serviceListener) {
		serviceListeners.remove(serviceListener);
	}

	/**
	 * Notifies listeners that this service has been started.
	 */
	protected void fireServiceStarted() {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceStarted(this);
		}
	}

	/**
	 * Notifies listeners that this service has been aborted due to a fatal
	 * exception.
	 * 
	 * @param throwable
	 *            The {@link Throwable} that caused this service to abort
	 */
	protected void fireServiceAborted(Throwable throwable) {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceAborted(this, throwable);
		}
	}

	/**
	 * Notifies listeners that this Service has been stopped. If an exception is
	 * the cause of stopping, a
	 * {@link ServiceListener#serviceAborted(Service, Throwable)} event was sent
	 * to listeners before this event.
	 */
	protected void fireServiceStopped() {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceStopped(this);
		}
	}

	protected void fireServiceThreadStarted(Thread thread) {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceThreadStarted(this, thread);
		}
	}

	protected void fireServiceThreadUpdated(Thread thread) {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceThreadUpdated(this, thread);
		}
	}

	protected void fireServiceThreadAborted(Thread thread, Throwable throwable) {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceThreadAborted(this, thread, throwable);
		}
	}

	protected void fireServiceThreadStopped(Thread thread) {
		for (ServiceListener serviceListener : serviceListeners) {
			serviceListener.serviceThreadStopped(this, thread);
		}
	}

	protected void setWaitPeriod(long waitPeriod) {
		this.waitPeriod = waitPeriod;
	}

	public void setThreadCount(int threadCount) {
		this.threadCount = threadCount;
	}

	/**
	 * Basic initialization of the service.
	 */
	public void init() {
		serviceListeners = new ArrayList<ServiceListener>();
		waitPeriod = 60 * 1000;
		threadCount = 1;
		threadGroup = new ThreadGroup(getName() + " Thread Group");
		threads = new ArrayList<ServiceThread>();
		threadCounter = 1;
	}

	/**
	 * Starts this service and the main thread.
	 */
	public void start() {
		synchronized (syncObject) {
			if (threads.size() == 0) {
				stopping = false;
				for (int threadIndex = 0; threadIndex < threadCount; threadIndex++) {
					ServiceThread newThread = new ServiceThread(threadGroup, this);
					newThread.start();
				}
			}
		}
	}

	/**
	 * Runs the service's main thread in a loop. The real work is done in
	 * {@link #work()}, this method just does some basic housekeeping (naming
	 * threads, checking mutexes, etc).
	 */
	public void run() {
		boolean stopped = false;
		Throwable throwable = null;

		synchronized (syncObject) {
			Thread.currentThread().setName(getName() + " Thread " + (threadCounter++));
			threads.add((ServiceThread) Thread.currentThread());
			fireServiceThreadStarted(Thread.currentThread());
			if (threads.size() == threadCount) {
				fireServiceStarted();
			}
		}

		while (!stopped) {

			try {
				work();
			} catch (Throwable t) {
				fireServiceThreadAborted(Thread.currentThread(), t);
				stopped = true;
				throwable = t;
				break;
			}

			/* check if we should stop */
			if (stopping) {
				stopped = true;
			}

			/* if not stopped, wait a minute for next run */
			if (!stopped) {
				try {
					setStatus("Idle for " + waitPeriod + " milliseconds.");
					synchronized (syncObject) {
						syncObject.wait(waitPeriod);
					}
				} catch (InterruptedException ie1) {
					stopped = true;
				}
			}

			/* check again - we might have been notify()ed. */
			if (stopping) {
				stopped = true;
			}
		}

		synchronized (syncObject) {
			threads.remove(Thread.currentThread());
			fireServiceThreadStopped(Thread.currentThread());
			syncObject.notify();
			if (threads.size() == 0) {
				if (throwable != null) {
					fireServiceAborted(throwable);
				} else {
					fireServiceStopped();
				}
			}
		}
	}

	/**
	 * This is where the real work of the AbstractService is done.
	 */
	protected void work() throws Throwable {
	}

	/**
	 * Stops the main thread of this service. This method will block until the
	 * thread has exited.
	 */
	public void stop() {
		synchronized (syncObject) {
			stopping = true;
			while (threads.size() > 0) {
				syncObject.notify();
				try {
					syncObject.wait(1000);
				} catch (InterruptedException ie1) {
				}
			}
		}
	}

	protected void setThreadStatus(String status) {
		setThreadStatus(Thread.currentThread(), status);
	}
	
	protected void setThreadStatus(Thread thread, String status) {
		if (thread instanceof ServiceThread) {
			((ServiceThread) thread).setStatus(status);
		}
	}
	
	public boolean isRunning() {
		return getRunningThreadCount() > 0;
	}

	public int getRunningThreadCount() {
		synchronized (syncObject) {
			return threads.size();
		}
	}

	public ServiceThread[] getThreads() {
		synchronized (syncObject) {
			return threads.toArray(new ServiceThread[threads.size()]);
		}
	}
	
	/**
	 * Releases all resources held by this service.
	 */
	public void destroy() {
		serviceListeners.clear();
		serviceListeners = null;
		threads.clear();
		threads = null;
	}

	public String toString() {
		return getName() + ": " + (isRunning() ? "" : "Not ") + "Running (" + getRunningThreadCount() + " Thread(s))";
	}
	
	protected void setStatus(String status) {
		Thread currentThread = Thread.currentThread();
		if (currentThread instanceof ServiceThread) {
			((ServiceThread) currentThread).setStatus(status);
			fireServiceThreadUpdated(currentThread);
		}
	}

}
