/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden </a>
 * @version $Id: ClientService.java 130 2006-02-18 18:27:11Z bombe $
 */
public class ClientService extends AbstractService {

	public static final String NAME = "Client Manager Service";
	private List<ClientListener> clientListeners;
	private Map<String, Client> clients;

	public String getName() {
		return NAME;
	}

	public void addClientListener(ClientListener clientListener) {
		if (!clientListeners.contains(clientListener)) {
			clientListeners.add(clientListener);
		}
	}

	public void removeClientListener(ClientListener clientListener) {
		clientListeners.remove(clientListener);
	}

	private void fireClientAdded(Client remoteClient) {
		for (ClientListener clientListener: clientListeners) {
			clientListener.clientAdded(remoteClient);
		}
	}

	private void fireClientUpdated(Client remoteClient) {
		for (ClientListener clientListener: clientListeners) {
			clientListener.clientUpdated(remoteClient);
		}
	}

	private void fireClientRemoved(Client remoteClient) {
		for (ClientListener clientListener: clientListeners) {
			clientListener.clientRemoved(remoteClient);
		}
	}

	public void addClient(Client remoteClient) {
		String clientId = remoteClient.getClientId();
		Client knownClient = getClient(clientId);
		if (knownClient != null) {
			if (remoteClient.getTimestamp() > knownClient.getTimestamp()) {
				synchronized (this) {
					clients.put(clientId, remoteClient);
				}
				fireClientUpdated(remoteClient);
			}
		} else {
			synchronized (this) {
				clients.put(clientId, remoteClient);
			}
			fireClientAdded(remoteClient);
		}
	}

	public void init() {
		super.init();
		clients = new HashMap<String, Client>();
		clientListeners = new ArrayList<ClientListener>();
	}

	public void destroy() {
		clients.clear();
		clients = null;
		super.destroy();
	}

	protected void work() {
		Iterator<Client> registeredClients = null;
		synchronized (this) {
			registeredClients = clients.values().iterator();
		}

		List<Client> clientsToRemove = new ArrayList<Client>();
		long now = System.currentTimeMillis();
		while (registeredClients.hasNext()) {
			Client client = registeredClients.next();
			if ((now - client.getTimestamp()) > 2 * 60 * 60 * 1000) {
				clientsToRemove.add(client);
			}
		}

		for (Client client: clientsToRemove) {
			synchronized (this) {
				clients.remove(client.getClientId());
			}
			fireClientRemoved(client);
		}
	}

	/**
	 * Returns the {@link Client} with the specified <code>clientId</code>.
	 * 
	 * @param clientId
	 *            The client's ID
	 * @return The {@link Client} with the specified ID, or <code>null</code>
	 */
	public synchronized Client getClient(String clientId) {
		return clients.get(clientId);
	}

	public synchronized int size() {
		return clients.size();
	}

	public synchronized Client[] getClients() {
		return clients.values().toArray(new Client[clients.size()]);
	}

}
