/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.EventListener;

/**
 * Interface for listeners that want to be notified on download events. Download
 * events are generated by the {@link DownloadService}.
 * 
 * @see freemule.application.DownloadService
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: DownloadListener.java 130 2006-02-18 18:27:11Z bombe $
 */
public interface DownloadListener extends EventListener {

	/**
	 * Notifies listeners that a download has been added to the list of
	 * downloads.
	 * 
	 * @param download
	 *            The download that was added
	 */
	public void downloadAdded(FreenetFile download);

	/**
	 * Notifies listeners that a download has been removed from the list of
	 * downloads.
	 * 
	 * @param download
	 *            The download that was removed
	 */
	public void downloadRemoved(FreenetFile download);

	/**
	 * Notifies listeners that a download has started. A "started" download is a
	 * download that is now actively looking for blocks.
	 * 
	 * @param download
	 *            The download that has been started.
	 */
	public void downloadStarted(FreenetFile download);

	/**
	 * Notifies listeners that a download has been stopped. A "stopped" download
	 * will not request further blocks from the network.
	 * 
	 * @param download
	 */
	public void downloadStopped(FreenetFile download);

	/**
	 * Notifies listeners that a download has collected all required blocks and
	 * is now moved to the incoming directory.
	 * 
	 * @param download
	 *            The download that is completing
	 */
	public void downloadCompleting(FreenetFile download);

	/**
	 * Notifies listeners that a download could not be moved to the incoming
	 * directory.
	 * 
	 * @param download
	 *            The download that should be moved to the incoming directory
	 * @param exception
	 *            The reason for the failure
	 */
	public void downloadCompletionFailed(FreenetFile download, Exception exception);

	/**
	 * Notifies listeners that a download is now completely downloaded and moved
	 * to the incoming directory.
	 * 
	 * @param download
	 *            The download that is complete
	 */
	public void downloadComplete(FreenetFile download);

	/**
	 * Notifies listeners that the manifest of a download was requested. The
	 * manifest holds further information about the download, such as length,
	 * number of blocks, etc.
	 * 
	 * @param download
	 *            The download to download the manifest for
	 */
	public void manifestDownloadStarted(FreenetFile download);

	/**
	 * Notifies listeners that the manifest of a download could not be retrieved
	 * from the network.
	 * <p>
	 * <strong>Note:</strong>not all kinds of exceptions are fatal. The reason
	 * for failure might be something as an
	 * {@link freenet.fcp.FcpRouteNotFoundException} in which case the download
	 * should simply be retried.
	 * 
	 * @param download
	 *            The download to download the manifest for
	 * @param exception
	 *            The exception that caused the download to fail
	 */
	public void manifestDownloadFailed(FreenetFile download, Exception exception);

	/**
	 * Notifies listeners that the download of a manifest completed
	 * successfully.
	 * 
	 * @param download
	 *            The download to download the manifest for
	 */
	public void manifestDownloadComplete(FreenetFile download);

	/**
	 * Notifies listeners that the download of a single block of data has
	 * started.
	 * 
	 * @param download
	 *            The download of the data block
	 * @param blockNumber
	 *            The number of the data block
	 */
	public void blockDownloadStarted(FreenetFile download, int blockNumber);

	/**
	 * Notifies listeners that the download of a data block has failed.
	 * <strong>Note:</strong>not all kinds of exceptions are fatal. The reason
	 * for failure might be something as an
	 * {@link freenet.fcp.FcpRouteNotFoundException} in which case the download
	 * should simply be retried.
	 * 
	 * @param download
	 *            The download of the data block
	 * @param blockNumber
	 *            The number of the data block
	 * @param exception
	 *            The exception that caused the download to fail
	 */
	public void blockDownloadFailed(FreenetFile download, int blockNumber, Exception exception);

	/**
	 * Notifies listeners that the download of a data block completed
	 * successfully.
	 * 
	 * @param download
	 *            The download of the data block
	 * @param blockNumber
	 *            The number of the data block
	 */
	public void blockDownloadComplete(FreenetFile download, int blockNumber);

	/**
	 * Notifies listeners that the loading of downloads from disk has started.
	 * 
	 * @param tempDir
	 *            The temporary directory downloads are stored in
	 */
	public void loadDownloadsStarted(String tempDir);

	/**
	 * Notifies listeners that the loading of downloads from disk has failed.
	 * 
	 * @param tempDir
	 *            The temporary directory downloads are stored in
	 * @param exception
	 *            The exception that caused loading to fail
	 */
	public void loadDownloadsFailed(String tempDir, Exception exception);

	/**
	 * Notifies listeners that the loading of downloads from disk has completed
	 * successfully.
	 * 
	 * @param tempDir
	 *            The temporary directory downloads are stored in
	 */
	public void loadDownloadsComplete(String tempDir);

	/**
	 * Notifies listeners that loading a single download from the properties
	 * file has started.
	 * 
	 * @param filename
	 *            The filename of the properties file
	 */
	public void loadDownloadStarted(String filename);

	/**
	 * Notifies listeners that laoding a single download from the properties
	 * file has failed.
	 * 
	 * @param filename
	 *            The filename of the properties file
	 * @param exception
	 *            The exception that caused loading to fail
	 */
	public void loadDownloadFailed(String filename, Exception exception);

	/**
	 * Notifies listeners that loading a single download from the properties
	 * file has completed successfully.
	 * 
	 * @param download
	 *            The download that was loaded
	 */
	public void loadDownloadComplete(String filename, FreenetFile download);

	/**
	 * Notifies listeners that saving the downloads to disk has started.
	 * 
	 * @param tempDir
	 *            The temporary directory that downloads are stored in
	 */
	public void saveDownloadsStarted(String tempDir);

	/**
	 * Notifies listeners that saving the downloads to disk has failed.
	 * 
	 * @param tempDir
	 *            The temporary directory that downloads are stored in
	 * @param exception
	 *            The exception that caused saving to fail
	 */
	public void saveDownloadsFailed(String tempDir, Exception exception);

	/**
	 * Notifies listeners that saving the downloads to disk has completed
	 * successfully.
	 * 
	 * @param tempDir
	 *            The temporary directory that downloads are stored in
	 */
	public void saveDownloadsComplete(String tempDir);

	/**
	 * Notifies listeners that saving a single download to a properties file has
	 * started.
	 * 
	 * @param download
	 *            The download to save
	 */
	public void saveDownloadStarted(FreenetFile download);

	/**
	 * Notifies listeners that saving a single download to a properties file has
	 * failed.
	 * 
	 * @param download
	 *            The download to save
	 * @param exception
	 *            The exception that caused saving to fail
	 */
	public void saveDownloadFailed(FreenetFile download, Exception exception);

	/**
	 * Notifies listeners that saving a single download to a properties file has
	 * completed successfully.
	 * 
	 * @param download
	 *            The download to save
	 */
	public void saveDownloadComplete(FreenetFile download);

	/**
	 * Notifies listeners that saving a data block to disk has started.
	 * 
	 * @param download
	 *            The download of the data block
	 * @param blockNumber
	 *            The number of the data block
	 */
	public void saveBlockStarted(FreenetFile download, int blockNumber);

	/**
	 * Notifies listeners that saving a data block to disk has failed.
	 * 
	 * @param download
	 *            The download of the data block
	 * @param blockNumber
	 *            The number of the data block
	 * @param exception
	 *            The exception that caused saving to fail
	 */
	public void saveBlockFailed(FreenetFile download, int blockNumber, Exception exception);

	/**
	 * Notifies listeners that saving a data block to disk has completed
	 * successfully.
	 * 
	 * @param download
	 *            The download of the data block
	 * @param blockNumber
	 *            The number of the data block
	 */
	public void saveBlockComplete(FreenetFile download, int blockNumber);

}
