/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import freemule.application.DownloadService;
import freemule.application.FileStatus;
import freemule.application.FreenetFile;
import freemule.gui.DownloadFilesTableModel.OldDownloadListener;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: DownloadFilesPanel.java 130 2006-02-18 18:27:11Z bombe $
 */
public class DownloadFilesPanel extends JPanel implements ListSelectionListener, OldDownloadListener {

	private JFrame mainFrame;
	private JTable downloadFilesTable;
	private DownloadFilesTableModel downloadFilesTableModel;
	private TableSorter downloadFilesTableSorter;
	private JPopupMenu popupMenu;

	private DownloadService downloadService;

	private Action removeOldDownloadsAction;
	private Action cancelDownloadAction;
	private Action stopDownloadAction;
	private Action resumeDownloadAction;
	private Action propertiesAction;

	public DownloadFilesPanel(JFrame mainFrame) {
		super(new BorderLayout(5, 5));
		this.mainFrame = mainFrame;
		setBorder(new EmptyBorder(5, 5, 5, 5));
		initializeActions();
		initializeComponents();
	}

	private void initializeActions() {
		removeOldDownloadsAction = new AbstractAction("Clean Up") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionRemoveOldDownloads();
			}
		};
		removeOldDownloadsAction.setEnabled(false);
		cancelDownloadAction = new AbstractAction("Cancel") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionCancelDownload();
			}
		};
		cancelDownloadAction.setEnabled(false);
		stopDownloadAction = new AbstractAction("Stop") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionStopDownload();
			}
		};
		stopDownloadAction.setEnabled(false);
		resumeDownloadAction = new AbstractAction("Resume") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionResumeDownload();
			}
		};
		resumeDownloadAction.setEnabled(false);
		propertiesAction = new AbstractAction("Properties") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionProperties();
			}
		};
		propertiesAction.setEnabled(false);
	}

	private void initializeComponents() {
		downloadFilesTableModel = new DownloadFilesTableModel();
		downloadFilesTableSorter = new TableSorter(downloadFilesTableModel);
		downloadFilesTable = new JTable(downloadFilesTableSorter);
		downloadFilesTableSorter.setTableHeader(downloadFilesTable.getTableHeader());
		downloadFilesTable.setDefaultRenderer(FreenetFile.class, new FileProgressBar());
		downloadFilesTable.getSelectionModel().addListSelectionListener(this);
		downloadFilesTable.addMouseListener(new MouseAdapter() {

			public void mousePressed(MouseEvent mouseEvent) {
				if (mouseEvent.isPopupTrigger()) {
					showPopupMenu(mouseEvent.getX(), mouseEvent.getY());
				}
			}

			public void mouseClicked(MouseEvent mouseEvent) {
				if (mouseEvent.isPopupTrigger()) {
					showPopupMenu(mouseEvent.getX(), mouseEvent.getY());
				}
			}
		});
		popupMenu = new JPopupMenu("Download");
		popupMenu.add(stopDownloadAction);
		popupMenu.add(resumeDownloadAction);
		popupMenu.add(cancelDownloadAction);
		popupMenu.add(propertiesAction);

		add(new JScrollPane(downloadFilesTable), BorderLayout.CENTER);
		add(createButtonPanel(), BorderLayout.PAGE_END);
	}

	private JPanel createButtonPanel() {
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 5, 5));
		buttonPanel.setBorder(new EmptyBorder(-5, -5, -5, -5));
		buttonPanel.add(new JButton(removeOldDownloadsAction));
		buttonPanel.add(new JButton(cancelDownloadAction));
		buttonPanel.add(new JButton(stopDownloadAction));
		buttonPanel.add(new JButton(resumeDownloadAction));
		buttonPanel.add(new JButton(propertiesAction));
		return buttonPanel;
	}

	public void setDownloadService(DownloadService downloadService) {
		this.downloadService = downloadService;
		downloadFilesTableModel.setDownloadService(downloadService);
	}

	protected void actionRemoveOldDownloads() {
		downloadFilesTableModel.removeOldDownloads();
		removeOldDownloadsAction.setEnabled(false);
	}

	protected void actionCancelDownload() {
		int selectedRow = downloadFilesTable.getSelectedRow();
		FreenetFile download = (FreenetFile) downloadFilesTableSorter.getValueAt(selectedRow, -1);
		if (download.getStatus() != FileStatus.complete) {
			downloadService.removeDownload(download);
		}
	}

	protected void actionStopDownload() {

	}

	protected void actionResumeDownload() {

	}

	protected void actionProperties() {
		int selectedRow = downloadFilesTable.getSelectedRow();
		FreenetFile download = (FreenetFile) downloadFilesTableSorter.getValueAt(selectedRow, -1);
		DownloadFilesPropertiesDialog propertiesDialog = new DownloadFilesPropertiesDialog(mainFrame, download);
		propertiesDialog.setVisible(true);
	}

	protected void showPopupMenu(int x, int y) {
		popupMenu.show(downloadFilesTable, x, y);
	}

	private void updateActions() {
		int[] selectedRows = downloadFilesTable.getSelectedRows();
		int currentDownloads = 0;
		for (int row: selectedRows) {
			if (!downloadFilesTableModel.isOldDownload(row)) {
				currentDownloads++;
			}
		}
		removeOldDownloadsAction.setEnabled(downloadFilesTableModel.hasOldDownloads());
		cancelDownloadAction.setEnabled(currentDownloads > 0);
		stopDownloadAction.setEnabled(currentDownloads > 0);
		resumeDownloadAction.setEnabled(currentDownloads > 0);
		propertiesAction.setEnabled(currentDownloads == 1);
	}

	/**
	 * {@inheritDoc}
	 */
	public void valueChanged(ListSelectionEvent listSelectionEvent) {
		updateActions();
	}

	/**
	 * {@inheritDoc}
	 */
	public void oldDownloadAdded(FreenetFile download) {
		updateActions();
	}

	/**
	 * {@inheritDoc}
	 */
	public void oldDownloadRemoved(FreenetFile download) {
		updateActions();
	}

}
