/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.util.EventListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.table.AbstractTableModel;

import freemule.application.DownloadListener;
import freemule.application.DownloadService;
import freemule.application.FileStatus;
import freemule.application.FreenetFile;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: DownloadFilesTableModel.java,v 1.2 2005/09/16 00:28:07 bombe
 *          Exp $
 */
public class DownloadFilesTableModel extends AbstractTableModel implements DownloadListener {

	private static final String[] columnNames = new String[] { "Name", "Size", "Progress", "Status" };
	private static final Class[] columnClasses = new Class[] { String.class, Integer.class, FreenetFile.class, String.class };

	private Set<OldDownloadListener> oldDownloadListeners = new HashSet<OldDownloadListener>();
	private DownloadService downloadService;
	private FreenetFile[] oldDownloads = new FreenetFile[0];
	private FreenetFile[] downloads;

	public void addOldDownloadListener(OldDownloadListener oldDownloadListener) {
		oldDownloadListeners.add(oldDownloadListener);
	}

	public void removeOldDownloadListener(OldDownloadListener oldDownloadListener) {
		oldDownloadListeners.remove(oldDownloadListener);
	}

	private void fireOldDownloadAdded(FreenetFile oldDownload) {
		for (OldDownloadListener oldDownloadListener: oldDownloadListeners) {
			oldDownloadListener.oldDownloadAdded(oldDownload);
		}
	}

	private void fireOldDownloadRemoved(FreenetFile oldDownload) {
		for (OldDownloadListener oldDownloadListener: oldDownloadListeners) {
			oldDownloadListener.oldDownloadRemoved(oldDownload);
		}
	}

	public void setDownloadService(DownloadService downloadService) {
		if (this.downloadService != null) {
			this.downloadService.removeDownloadListener(this);
		}
		this.downloadService = downloadService;
		if (this.downloadService != null) {
			this.downloadService.addDownloadListener(this);
		}
		updateDownloads();
		fireTableDataChanged();
	}

	public void removeOldDownloads() {
		int numberOfOldDownloads = oldDownloads.length;
		for (int downloadIndex = 0; downloadIndex < numberOfOldDownloads; downloadIndex++) {
			fireOldDownloadRemoved(oldDownloads[downloadIndex]);
		}
		oldDownloads = new FreenetFile[0];
		updateDownloads();
		fireTableRowsDeleted(0, numberOfOldDownloads - 1);
	}

	public boolean hasOldDownloads() {
		return oldDownloads.length > 0;
	}

	public boolean isOldDownload(int downloadIndex) {
		return (downloads[downloadIndex].getStatus() == FileStatus.complete);
	}

	private synchronized void updateDownloads() {
		if (downloadService == null) {
			downloads = null;
		} else {
			FreenetFile[] newDownloads = downloadService.getDownloads();
			downloads = new FreenetFile[oldDownloads.length + newDownloads.length];
			System.arraycopy(oldDownloads, 0, downloads, 0, oldDownloads.length);
			System.arraycopy(newDownloads, 0, downloads, oldDownloads.length, newDownloads.length);
		}
	}

	public synchronized int getDownloadIndex(FreenetFile wantedDownload) {
		for (int index = 0, count = downloads.length; index < count; index++) {
			if (downloads[index].equals(wantedDownload)) {
				return index;
			}
		}
		return -1;
	}

	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		if (downloads == null) {
			return 0;
		}
		return downloads.length;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return (columnIndex == 0);
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	public Class< ? > getColumnClass(int columnIndex) {
		return columnClasses[columnIndex];
	}

	/**
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (downloads == null) {
			return null;
		}
		FreenetFile download = downloads[rowIndex];
		switch (columnIndex) {
		case -1:
			return download;
		case 0:
			return download.getName();
		case 1:
			return String.valueOf(download.getLength());
		case 2:
			return download;
		case 3:
			if (download.getStatus() == FileStatus.complete) {
				return "Complete";
			}
			if (download.getStatus() == FileStatus.deactivated) {
				return "Stopped";
			}
			return "Downloading";
		}
		return null;
	}

	/**
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (downloads == null) {
			return;
		}
		downloads[rowIndex].setName((String) aValue);
		downloadService.saveTempDirectory();
		fireTableCellUpdated(rowIndex, columnIndex);
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int columnIndex) {
		return columnNames[columnIndex];
	}

	//
	// INTERFACE DownloadListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void downloadAdded(FreenetFile download) {
		updateDownloads();
		fireTableRowsInserted(downloads.length - 1, downloads.length - 1);
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadRemoved(FreenetFile download) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsDeleted(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadStarted(FreenetFile download) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadStopped(FreenetFile download) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadComplete(FreenetFile download) {
		FreenetFile[] tempOldDownloads = new FreenetFile[oldDownloads.length + 1];
		System.arraycopy(oldDownloads, 0, tempOldDownloads, 0, oldDownloads.length);
		tempOldDownloads[oldDownloads.length] = download;
		oldDownloads = tempOldDownloads;
		updateDownloads();
		fireOldDownloadAdded(download);
		fireTableDataChanged();
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadComplete(String filename, FreenetFile download) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadFailed(String filename, Exception e) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadComplete(FreenetFile download) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveBlockComplete(FreenetFile download, int blockNumber) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadCompleting(FreenetFile download) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void downloadCompletionFailed(FreenetFile download, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestDownloadStarted(FreenetFile download) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestDownloadFailed(FreenetFile download, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestDownloadComplete(FreenetFile download) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockDownloadStarted(FreenetFile download, int blockNumber) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockDownloadFailed(FreenetFile download, int blockNumber, Exception exception) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockDownloadComplete(FreenetFile download, int blockNumber) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadsStarted(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadsFailed(String tempDir, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadsComplete(String tempDir) {
		updateDownloads();
		fireTableDataChanged();
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadDownloadStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadsStarted(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadsFailed(String tempDir, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadsComplete(String tempDir) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadStarted(FreenetFile download) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveDownloadFailed(FreenetFile download, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveBlockStarted(FreenetFile download, int blockNumber) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveBlockFailed(FreenetFile download, int blockNumber, Exception exception) {
		int downloadPosition = getDownloadIndex(download);
		updateDownloads();
		fireTableRowsUpdated(downloadPosition, downloadPosition);
	}

	public static interface OldDownloadListener extends EventListener {

		public void oldDownloadAdded(FreenetFile download);

		public void oldDownloadRemoved(FreenetFile download);

	}

}
