/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import freemule.application.AbstractService;
import freemule.application.Service;
import freemule.application.ServiceListener;
import freemule.application.ServiceThread;

/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: ServicesTreeModel.java 130 2006-02-18 18:27:11Z bombe $
 */
public class ServicesTreeModel implements TreeModel, ServiceListener {

	private List<TreeModelListener> treeModelListeners;

	private final Object root = new Object();

	private final AbstractService[] services;

	public ServicesTreeModel(AbstractService[] services) {
		this.services = services;
		this.treeModelListeners = new ArrayList<TreeModelListener>();
		if (services != null) {
			for (Service service: services) {
				((AbstractService) service).addServiceListener(this);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getRoot() {
		return root;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getChild(Object parent, int index) {
		if (parent == root) {
			if (services == null) {
				return null;
			}
			return services[index];
		} else if (parent instanceof AbstractService) {
			ServiceThread[] serviceThreads = ((AbstractService) parent).getThreads();
			return serviceThreads[index];
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getChildCount(Object parent) {
		if (parent == root) {
			if (services == null) {
				return 0;
			}
			return services.length;
		} else if (parent instanceof AbstractService) {
			ServiceThread[] serviceThreads = ((AbstractService) parent).getThreads();
			return serviceThreads.length;
		}
		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isLeaf(Object node) {
		return (node instanceof ServiceThread);
	}

	/**
	 * {@inheritDoc}
	 */
	public void valueForPathChanged(TreePath path, Object newValue) {
		fireTreeNodesChanged(path.getParentPath(), new int[] { getIndexOfChild(path.getParentPath().getLastPathComponent(), path.getLastPathComponent()) }, new Object[] { newValue });
	}

	/**
	 * {@inheritDoc}
	 */
	public int getIndexOfChild(Object parent, Object child) {
		if ((parent == null) || (child == null)) {
			return -1;
		}
		if (parent == root) {
			for (int serviceIndex = 0, serviceCount = services.length; serviceIndex < serviceCount; serviceIndex++) {
				if (child == services[serviceIndex]) {
					return serviceIndex;
				}
			}
		} else if (parent instanceof AbstractService) {
			ServiceThread[] serviceThreads = ((AbstractService) parent).getThreads();
			for (int threadIndex = 0, threadCount = serviceThreads.length; threadIndex < threadCount; threadIndex++) {
				if (child == serviceThreads[threadIndex]) {
					return threadIndex;
				}
			}
		}
		return -1;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addTreeModelListener(TreeModelListener treeModelListener) {
		treeModelListeners.add(treeModelListener);
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeTreeModelListener(TreeModelListener treeModelListener) {
		treeModelListeners.remove(treeModelListener);
	}

	protected void fireTreeNodesChanged(TreePath pathToParentOfChangedNode, int[] indicesOfChangedNodes, Object[] changedNodes) {
		TreeModelEvent treeModelEvent = new TreeModelEvent(this, pathToParentOfChangedNode, indicesOfChangedNodes, changedNodes);
		for (TreeModelListener treeModelListener : treeModelListeners) {
			treeModelListener.treeNodesChanged(treeModelEvent);
		}
	}

	protected void fireTreeNodesInserted(TreePath pathToParentOfInsertedNodes, int[] indicesOfNewNodes, Object[] newNodes) {
		TreeModelEvent treeModelEvent = new TreeModelEvent(this, pathToParentOfInsertedNodes, indicesOfNewNodes, newNodes);
		for (TreeModelListener treeModelListener : treeModelListeners) {
			treeModelListener.treeNodesInserted(treeModelEvent);
		}
	}

	protected void fireTreeNodesRemoved(TreePath pathToParentOfRemovedNodes, int[] indicesOfRemovedNodes) {
		TreeModelEvent treeModelEvent = new TreeModelEvent(this, pathToParentOfRemovedNodes, indicesOfRemovedNodes, null);
		for (TreeModelListener treeModelListener : treeModelListeners) {
			treeModelListener.treeNodesChanged(treeModelEvent);
		}
	}

	protected void fireTreeStructureChanged(TreePath pathOfParentOfChanges) {
		TreeModelEvent treeModelEvent = new TreeModelEvent(this, pathOfParentOfChanges, null, null);
		for (TreeModelListener treeModelListener : treeModelListeners) {
			treeModelListener.treeNodesChanged(treeModelEvent);
		}
	}

	//
	// INTERFACE ServiceListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void serviceStarted(Service service) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceAborted(Service service, Throwable throwable) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceStopped(Service service) {
		TreePath pathToRoot = new TreePath(new Object[] { root });
		int serviceIndex = getIndexOfChild(root, service);
		fireTreeNodesChanged(pathToRoot, new int[] { serviceIndex }, new Object[] { service });
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadStarted(Service service, Thread thread) {
		TreePath pathToService = new TreePath(new Object[] { root, service });
		int newIndex = getChildCount(service);
		fireTreeNodesInserted(pathToService, new int[] { newIndex }, new Object[] { thread });
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadUpdated(Service service, Thread thread) {
		TreePath pathToService = new TreePath(new Object[] { root, service });
		int changedIndex = getIndexOfChild(service, thread);
		fireTreeNodesChanged(pathToService, new int[] { changedIndex }, new Object[] { thread });
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadAborted(Service service, Thread thread, Throwable throwable) {
		TreePath pathToService = new TreePath(new Object[] { root, service });
		int removedIndex = getIndexOfChild(service, thread);
		fireTreeNodesRemoved(pathToService, new int[] { removedIndex });
		fireTreeStructureChanged(pathToService);
	}

	/**
	 * {@inheritDoc}
	 */
	public void serviceThreadStopped(Service service, Thread thread) {
		TreePath pathToService = new TreePath(new Object[] { root, service });
		int removedIndex = getIndexOfChild(service, thread);
		fireTreeNodesRemoved(pathToService, new int[] { removedIndex });
		fireTreeStructureChanged(pathToService);
	}

}
