/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * A FilePartInputStream is an {@link FileInputStream} that is restricted to a
 * specified part of a {@link File}.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FilePartInputStream.java 130 2006-02-18 18:27:11Z bombe $
 */
public class FilePartInputStream extends FileInputStream {

	/**
	 * The number of bytes that remain in this stream.
	 */
	private long length;

	/**
	 * Creates a new FilePartInputStream with the specified parameters.
	 * 
	 * @param filename
	 *            The name of the file to read
	 * @param offset
	 *            The offset from which to start
	 * @param length
	 *            The size of the stream
	 * @throws FileNotFoundException
	 *             if <code>file</code> can not be found
	 * @throws IOException
	 *             if an I/O exception occurs
	 * @throws IllegalArgumentException
	 *             if the given <code>offset</code> and <code>length</code>
	 *             are too large for <code>file</code>
	 */
	public FilePartInputStream(String filename, long offset, long length) throws FileNotFoundException, IOException {
		this(new File(filename), offset, length);
	}

	/**
	 * Creates a new FilePartInputStream with the specified parameters.
	 * 
	 * @param file
	 *            The file to read
	 * @param offset
	 *            The offset from which to start
	 * @param length
	 *            The size of the stream
	 * @throws FileNotFoundException
	 *             if <code>file</code> can not be found
	 * @throws IOException
	 *             if an I/O exception occurs
	 * @throws IllegalArgumentException
	 *             if the given <code>offset</code> and <code>length</code>
	 *             are too large for <code>file</code>
	 */
	public FilePartInputStream(File file, long offset, long length) throws FileNotFoundException, IOException {
		super(file);
		if (file.length() < offset + length) {
			throw new IllegalArgumentException("offset + length > filesize");
		}
		long skipped = 0;
		while (skipped < offset) {
			long s = skip(offset - skipped);
			skipped += s;
		}
		this.length = length;
	}

	/**
	 * Returns the number of bytes that can be read from this stream without
	 * causing {@link #read()} to block.
	 * 
	 * @return The number of bytes that can be read without blocking
	 * @throws IOException
	 *             if an I/O exception occurs
	 */
	@Override
	public int available() throws IOException {
		return (int) Math.min(Integer.MAX_VALUE, Math.min(length, super.available()));
	}

	/**
	 * Reads a single byte from the stream. If the end of the stream is reached,
	 * <code>-1</code> is returned. This method will block until a byte is
	 * read, or the end of the stream is reached.
	 * 
	 * @return A single byte from the stream, or <code>-1</code> if the end of
	 *         the stream is reached
	 * @throws IOException
	 *             if an I/O exception occurs
	 */
	@Override
	public int read() throws IOException {
		if (length <= 0) {
			return -1;
		}
		int r = super.read();
		if (r != -1) {
			length--;
		}
		return r;
	}

	/**
	 * This method reads as many bytes from the stream as are needed to fill
	 * <code>buffer</code>. It will block until at least one byte has been
	 * read, or the end of the stream has been reached. Calling this method is
	 * equivalent to calling
	 * {@link #read(byte[], int, int) read(buffer, 0, buffer.length)}.
	 * 
	 * @param buffer
	 *            The buffer to fill
	 * @return The number of bytes that have been read, or <code>-1</code> if
	 *         the end of the stream has been reached
	 * @throws IOException
	 *             if an I/O exception occurs
	 */
	@Override
	public int read(byte[] buffer) throws IOException {
		return read(buffer, 0, buffer.length);
	}

	/**
	 * This method tries to read as many bytes from the stream as are needed to
	 * fill <code>bufferLength</code> bytes in <code>buffer</code>,
	 * beginning at <code>bufferOffset</code>. It will block until at least a
	 * single byte has been read, or the end of the stream is reached. If the
	 * buffer is not large enough to hold the requested amount of data, an
	 * {@link ArrayIndexOutOfBoundsException} is thrown.
	 * 
	 * @param buffer
	 *            The buffer to fill
	 * @param bufferOffset
	 *            The offset from which to begin filling the buffer
	 * @param bufferLength
	 *            The maximum number of bytes to read
	 * @return The number of bytes that have been read, or <code>-1</code> if
	 *         the end of the stream has been reached
	 * @throws ArrayIndexOutOfBoundsException
	 *             if <code>buffer</code> is not large enough
	 * @throws IOException
	 *             if an I/O exception occurs
	 */
	@Override
	public int read(byte[] buffer, int bufferOffset, int bufferLength) throws IOException {
		if (length <= 0) {
			return -1;
		}
		int read = super.read(buffer, bufferOffset, (int) Math.min(Integer.MAX_VALUE, Math.min(bufferLength, length)));
		if (read != -1) {
			length -= read;
		}
		return read;
	}

}
