/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.util;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Helper class for conversion from different data formats to a {@link String}
 * containing hexadecimal numbers, and vice versa.
 * 
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: Hex.java 130 2006-02-18 18:27:11Z bombe $
 */
public class Hex {

	/**
	 * The 16 hexadecimal digits.
	 */
	private static char[] hexDigits = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

	/**
	 * Converts a byte array to a {@link String} containing hexadecimal digits.
	 * 
	 * @see #toBytes(String)
	 * @param data
	 *            The data to convert
	 * @return The converted data
	 */
	public static String toHex(byte[] data) {
		StringBuffer hexString = new StringBuffer(data.length << 1);
		for (int i = 0, s = data.length; i < s; i++) {
			int value = data[i] & 0xff;
			hexString.append(hexDigits[value / 16]).append(hexDigits[value & 0x0f]);
		}
		return hexString.toString();
	}

	/**
	 * Converts a single number to hexadecimal format.
	 * 
	 * @see #toLong(String)
	 * @param number
	 *            The number to convert
	 * @return The converted number
	 */
	public static String toHex(long number) {
		StringBuffer hexString = new StringBuffer(16);
		while (number != 0) {
			int value = (int) (number & 0x0f);
			hexString.insert(0, hexDigits[value & 0x0f]);
			number >>>= 4;
		}
		if (hexString.length() == 0) {
			hexString.append("0");
		}
		return hexString.toString();
	}

	/**
	 * Converts the numbers in the {@link List} to hexadecimal format and
	 * returns a list. The single entries of <code>list</code> are
	 * space-separated.
	 * 
	 * @see #toList(String)
	 * @param list
	 *            The numbers to convert
	 * @return The converted list
	 */
	public static String toHex(List<Integer> list) {
		StringBuffer listResult = new StringBuffer();
		for (int listEntry : list) {
			if (listResult.length() > 0) {
				listResult.append(',');
			}
			listResult.append(toHex(listEntry));
		}
		return listResult.toString();
	}

	/**
	 * Converts the indexes in the {@link BitSet} to a list of hexadecimal
	 * numbers.
	 * 
	 * @see #toBitSet(String)
	 * @param bitSet
	 *            The {@link BitSet} to convert
	 * @return The converted list
	 */
	public static String toHex(BitSet bitSet) {
		StringBuffer listResult = new StringBuffer();
		for (int bitIndex = 0, bitCount = bitSet.length(); bitIndex < bitCount; bitIndex++) {
			if (bitSet.get(bitIndex)) {
				if (listResult.length() > 0) {
					listResult.append(',');
				}
				listResult.append(toHex(bitIndex));
			}
		}
		return listResult.toString();
	}

	/**
	 * Converts the hexadecimal representation of a number into that number.
	 * 
	 * @see #toHex(long)
	 * @param hex
	 *            The hexadecimal number
	 * @return The converted number
	 */
	public static long toLong(String hex) {
		long value = 0;
		for (int index = 0, count = hex.length(); index < count; index++) {
			char c = hex.charAt(index);
			value = (value << 4) | ((c - 48) - ((c > 57) ? 7 : 0) - ((c > 89) ? 32 : 0));
		}
		return value;
	}

	/**
	 * Converts the hexadecimal representation of a byte array into that byte
	 * array.
	 * 
	 * @see #toHex(byte[])
	 * @param hex
	 *            The hexadecimal representation of a byte array
	 * @return The converted byte array
	 */
	public static byte[] toBytes(String hex) {
		byte[] result = new byte[hex.length() / 2];
		for (int index = 0, size = hex.length(); index < size; index += 2) {
			result[index / 2] = (byte) toLong(hex.substring(index, index + 2));
		}
		return result;
	}

	/**
	 * Converts the hexadecimal representation of a {@link List} back into that
	 * {@link List}.
	 * 
	 * @see #toHex(List)
	 * @param listText
	 *            The hexadecimal representation of a {@link List}
	 * @return The converted {@link List}
	 */
	public static List<Integer> toList(String listText) {
		if ((listText == null) || (listText.length() == 0)) {
			return new ArrayList<Integer>();
		}
		List<Integer> list = new ArrayList<Integer>();
		StringTokenizer listTokenizer = new StringTokenizer(listText, ",");
		while (listTokenizer.hasMoreTokens()) {
			String listToken = listTokenizer.nextToken();
			list.add((int) toLong(listToken));
		}
		return list;
	}

	/**
	 * Converts the hexadecimal representation of a {@link BitSet} back into
	 * that {@link BitSet}.
	 * 
	 * @see #toHex(BitSet)
	 * @param bitSetText
	 *            The hexadecimal representation of a {@link BitSet}
	 * @return The converted {@link BitSet}
	 */
	public static BitSet toBitSet(String bitSetText) {
		if ((bitSetText == null) || (bitSetText.length() == 0)) {
			return new BitSet();
		}
		BitSet bitSet = new BitSet();
		StringTokenizer listTokenizer = new StringTokenizer(bitSetText, ",");
		while (listTokenizer.hasMoreTokens()) {
			String listToken = listTokenizer.nextToken();
			bitSet.set((int) toLong(listToken));
		}
		return bitSet;
	}

}
