/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

/**
 * Contains URIs for data and check blocks, used in Splitfile handling.
 * 
 * @see freenet.fcp.FcpFecSegmentFile
 * @author David Roden <droden@gmail.com>
 * @version $Id: BlockMap.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class BlockMap {

	private int dataBlockCount;
	private int checkBlockCount;

	/**
	 * The data block URIs.
	 */
	private String[] dataBlocks;

	/**
	 * The check block URIs.
	 */
	private String[] checkBlocks;

	/**
	 * Creates a new BlockMap with the specified data and check blocks.
	 * 
	 * @param dataBlockCount
	 *            The number of data blocks
	 * @param checkBlockCount
	 *            The number of check blocks
	 */
	public BlockMap(int dataBlockCount, int checkBlockCount) {
		this.dataBlockCount = dataBlockCount;
		this.checkBlockCount = checkBlockCount;
		dataBlocks = new String[dataBlockCount];
		checkBlocks = new String[checkBlockCount];
	}

	/**
	 * @return Returns the checkBlockCount.
	 */
	public int getCheckBlockCount() {
		return checkBlockCount;
	}

	/**
	 * @return Returns the dataBlockCount.
	 */
	public int getDataBlockCount() {
		return dataBlockCount;
	}

	/**
	 * Sets the URI of the specified data block.
	 * 
	 * @param number
	 *            The number of the data block
	 * @param key
	 *            The URI of the data block
	 */
	public void setDataBlock(int number, String key) {
		if (number >= dataBlocks.length) {
			setCheckBlock(number - dataBlocks.length, key);
			return;
		}
		dataBlocks[number] = key;
	}

	/**
	 * Sets the URI of the specified check block.
	 * 
	 * @param number
	 *            The number of the check block
	 * @param key
	 *            The URI of the check block
	 */
	public void setCheckBlock(int number, String key) {
		checkBlocks[number] = key;
	}

	/**
	 * Returns the URI of the specified data block.
	 * 
	 * @param number
	 *            The number of the data block
	 * @return The URI of the specified data block
	 */
	public String getDataBlock(int number) {
		if (number >= dataBlocks.length) {
			return getCheckBlock(number - dataBlocks.length);
		}
		return dataBlocks[number];
	}

	/**
	 * Returns the URI of the specified check block.
	 * 
	 * @param number
	 *            The number of the check block
	 * @return The URI of the specified check block
	 */
	public String getCheckBlock(int number) {
		return checkBlocks[number];
	}

	/**
	 * Converts this BlockMap into an array of bytes. The resulting byte array
	 * can be written to an {@link java.io.OutputStream}.
	 * 
	 * @return An array of bytes containing a textual representation of this
	 *         BlockMap
	 */
	public byte[] toByteArray() {
		ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
		PrintWriter writer = new PrintWriter(new OutputStreamWriter(byteOutput));
		writer.println("BlockMap");
		for (int index = 0, size = dataBlocks.length; index < size; index++) {
			writer.println("Block." + FcpCommand.encodeHex(index) + "=" + dataBlocks[index]);
		}
		for (int index = 0, size = checkBlocks.length; index < size; index++) {
			writer.println("Check." + FcpCommand.encodeHex(index) + "=" + checkBlocks[index]);
		}
		writer.println("EndMessage");
		writer.close();
		try {
			byteOutput.close();
		} catch (IOException ioe1) {
		}
		return byteOutput.toByteArray();
	}

}
