/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.IOException;
import java.io.OutputStream;

/**
 * FCP <code>ClientHello</code> command implementation. The ClientHello
 * command is used to determine certain node parameters, such as the version and
 * the protocol it uses. It is also a useful way to test for the presence of a
 * Freenet node.
 * 
 * @see FcpClientInfo
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpClientHello.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class FcpClientHello extends FcpCommand {

	/**
	 * The protocol this node talks.
	 */
	private String protocol;

	/**
	 * The node's identification.
	 */
	private String node;

	/**
	 * The highest build number the node seen.
	 */
	private String highestSeenBuild;

	/**
	 * The maximum file size this node can handle.
	 */
	private int maxFileSize;

	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return "ClientHello";
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) {
		if ("NodeHello".equals(fcpMessage.getName())) {
			protocol = fcpMessage.getParameter("Protocol");
			node = fcpMessage.getParameter("Node");
			highestSeenBuild = fcpMessage.getParameter("HighestSeenBuild");
			maxFileSize = (int) parseHex(fcpMessage.getParameter("MaxFileSize"));
			complete = true;
		}
	}

	/**
	 * Returns the highest build number the node has seen.
	 * 
	 * @return The highest build number the node has seen
	 */
	public String getHighestSeenBuild() {
		return highestSeenBuild;
	}

	/**
	 * Returns the maximum file size this node can handle.
	 * 
	 * @return The maximum file size this node can handle
	 */
	public int getMaxFileSize() {
		return maxFileSize;
	}

	/**
	 * Returns the node&rsquo;s internal version identification string.
	 * 
	 * @return The node&rsquo;s internal version identification string
	 */
	public String getNode() {
		return node;
	}

	/**
	 * Returns the protocol the node talks.
	 * 
	 * @return The protocol the node talks
	 */
	public String getProtocol() {
		return protocol;
	}

}
