/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

/**
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpFecEncodeSegment.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class FcpFecEncodeSegment extends FcpCommand {

	private static final int BUFFER_SIZE = 128 * 1024;

	private List/* <Integer> */requestedBlocks;
	private SegmentHeader segmentHeader;
	private InputStream dataInput;
	private long dataLength;
	private int checkBlockCount;
	private int checkBlockSize;
	private int dataPointer;
	
	private File tempFile;
	private OutputStream checkBlockOutput;

	public FcpFecEncodeSegment(SegmentHeader segmentHeader, byte[] data) {
		this(null, segmentHeader, data);
	}

	public FcpFecEncodeSegment(SegmentHeader segmentHeader, InputStream dataInput, long dataLength) {
		this(null, segmentHeader, dataInput, dataLength);
	}

	public FcpFecEncodeSegment(List/* <Integer> */requestedBlocks, SegmentHeader segmentHeader, byte[] data) {
		this(requestedBlocks, segmentHeader, new ByteArrayInputStream(data), data.length);
	}

	public FcpFecEncodeSegment(List/* <Integer> */requestedBlocks, SegmentHeader segmentHeader, InputStream dataInput, long dataLength) {
		this.requestedBlocks = requestedBlocks;
		this.segmentHeader = segmentHeader;
		this.dataInput = dataInput;
		this.dataLength = dataLength;
	}

	/**
	 * @see freenet.fcp.FcpCommand#getName()
	 */
	public String getName() {
		return "FECEncodeSegment";
	}

	public int getCheckBlockCount() {
		return checkBlockCount;
	}

	public int getCheckBlockSize() {
		return checkBlockSize;
	}

	/**
	 * As soon as this InputStream is closed, the file backing it will be gone,
	 * i.e. this method can only be called once!
	 * 
	 * @return An InputStream containing the check block data
	 */
	public InputStream getCheckBlockData() {
		try {
			return new TemporaryFileInputStream(tempFile);
		} catch (FileNotFoundException e) {
			// ? FIXME
		}
		return null;
	}

	/**
	 * @see freenet.fcp.FcpCommand#writeCommand(java.io.OutputStream)
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		if ((requestedBlocks != null) && (requestedBlocks.size() > 0)) {
			outputStream.write(("RequestedList=" + encodeList(requestedBlocks) + "\r\n").getBytes());
		}
		int segmentLength = segmentHeader.getBlockCount() * segmentHeader.getBlockSize();
		byte[] segmentHeaderData = segmentHeader.toByteArray();
		// the segment header is sent as meta-data! gj could have mentioned
		// that.
		outputStream.write(("MetadataLength=" + encodeHex(segmentHeaderData.length) + "\r\n").getBytes());
		outputStream.write(("DataLength=" + encodeHex(segmentHeaderData.length + segmentLength) + "\r\n").getBytes());
		outputStream.write(("Data\r\n").getBytes());
		outputStream.write(segmentHeaderData);
		copyStream(outputStream, dataInput, Math.min(dataLength, segmentLength));
		long zeroFillBytes = segmentLength - dataLength;
		if (zeroFillBytes > 0) {
			byte[] temp = new byte[(int) Math.min(zeroFillBytes, BUFFER_SIZE)];
			while (zeroFillBytes > 0) {
				outputStream.write(temp, 0, (int) Math.min(zeroFillBytes, BUFFER_SIZE));
				zeroFillBytes -= Math.min(zeroFillBytes, BUFFER_SIZE);
			}
		}
		return false;
	}

	/**
	 * @see freenet.fcp.FcpCommand#handleFcpMessage(freenet.fcp.FcpMessage)
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) throws FcpException {
		String fcpMessageName = fcpMessage.getName();
		if (fcpMessageName.equals("BlocksEncoded")) {
			checkBlockCount = (int) parseHex(fcpMessage.getParameter("BlockCount"));
			checkBlockSize = (int) parseHex(fcpMessage.getParameter("BlockSize"));
			dataPointer = 0;
			try {
				tempFile = File.createTempFile("freeMule.", ".check");
				checkBlockOutput = new FileOutputStream(tempFile);
			} catch (IOException e) {
				// FIXME
			}
		} else if (fcpMessageName.equals("Restarted")) {
			try {
				checkBlockOutput.close();
				checkBlockOutput = new FileOutputStream(tempFile);
			} catch (IOException ioe1) {
			}
		} else if (fcpMessageName.equals("DataChunk")) {
			byte[] checkBlockData = fcpMessage.getData();
			try {
				checkBlockOutput.write(checkBlockData);
			} catch (IOException ioe1) {
			}
			dataPointer += checkBlockData.length;
			complete = (dataPointer == (checkBlockCount * checkBlockSize));
			if (complete) {
				try {
					checkBlockOutput.close();
				} catch (IOException e) {
				}
			}
		}
	}

	public static class TemporaryFileInputStream extends FileInputStream {

		private File tempFile;

		public TemporaryFileInputStream(File tempFile) throws FileNotFoundException {
			super(tempFile);
			this.tempFile = tempFile;
		}

		public void close() {
			tempFile.delete();
		}

	}

}
